#!/usr/bin/env python

"""
Reads in source files and rebuilds innerHTML strings from html source files.
For example:
  in srcfile.js:
    //---filterTable.html:---
    somediv.innerHTML = '<input...';
    
then running 'python buildinnerHTML.py srcfile.js' will replace the '<input...'
string with the compactified contents of 'filterTable.html'.  The next line
after the comment string must have single quotes.

The default regular expression that finds includes is '//---(.+?):---', and can
be changed with the -e "REGEXP" or --regexp="REGEXP" options.  Include a
parenthesis group to match the filename.

Alex Leone, 2009-05-30
"""

from optparse import OptionParser
import re
from shutil import move
from time import time

def compactifyHTML(filename):
    """
    Strips each line, removes newlines, and escapes backslashes & single quotes
    """
    f = open(filename, 'r')
    flines = f.read().splitlines()
    f.close()
    flines = [line.strip() for line in flines]
    return ''.join(flines).replace("\\", "\\\\").replace("'", "\\'")  

def main():
    usage = 'usage: %prog [options] srcfiles...'
    parser = OptionParser(usage)
    parser.add_option('-b', '--backup', dest='backup', action='store_true',
                      help='Saves a backup file.')
    parser.add_option('-e', '--regexp', dest='includere',
                      default='//---(.+?):---',
                      help='The regexp that finds files to include.  '
                           'See python regexp syntax.  [default: "%default"]')
    parser.add_option('-v', '--verbose', dest='verbose', action='store_true')
    (options, args) = parser.parse_args()
    if len(args) < 1:
        parser.error('specify a source file')
    try:
        incre = re.compile(options.includere)
    except Exception as e:
        parser.error('invalid regexp %r: %s' % (options.includere, e))
    if '(' not in options.includere:
        parser.error('invalid regexp %r: Needs at least one parenthesis group' \
                     % options.includere)
    for filename in args:
        if options.verbose:
            print('reading %r...' % filename)
        srcf = open(filename, 'r')
        sflines = srcf.read().splitlines()
        srcf.close()
        if options.backup:
            bkfilename = '%s.bk%d' % (filename, int(time()))
            if options.verbose:
                print('   saving backup %r...' % bkfilename) 
            move(filename, bkfilename)
        for i, line in enumerate(sflines):
            match = incre.search(line)
            if match == None:
                continue
            refilename = match.group(1)
            if options.verbose:
                print('   found include replacement on line %d: %r' % (
                       i, match.group(0)))
                print('   reading %r...' % refilename)
            nline = sflines[i + 1]
            beforerep = nline[0:nline.find("'") + 1]
            afterrep = nline[nline.rfind("'"):]
            sflines[i + 1] = beforerep + compactifyHTML(refilename) + afterrep
        srcfout = open(filename, 'w')
        srcfout.write('\n'.join(sflines))
        srcfout.close()

if __name__ == '__main__':
    main()
